@@ -1,7 +1,13 @@
 from pathlib import Path
 import json
 import re
+import urllib.parse
+
 import requests
+import nbformat
+from nbconvert.exporters import MarkdownExporter
+
+from ._preprocesors import MarkdownPreprocessor, NoExecuteDataFramePreprocessor
 
 
 class Publish:
@@ -14,8 +20,9 @@ class Publish:
     IMAGE_TYPES = {'png', 'gif', 'jpeg', 'jpg', 'tiff'}
     
 
-    def __init__(self, filename, integration_token, pub_name, dataframe_image, 
-                 title, tags, publish_status, notify_followers, license, canonical_url):
+    def __init__(self, filename, integration_token, pub_name, title, tags, 
+                 publish_status, notify_followers, license, canonical_url,
+                 chrome_path, download_markdown):
         self.filename = Path(filename)
         self.integration_token = self.get_integration_token(integration_token)
         self.pub_name = pub_name
@@ -25,23 +32,33 @@ class Publish:
         self.notify_followers = str(notify_followers).lower()
         self.license = license
         self.canonical_url = canonical_url
-        self.nb_dir = self.filename.resolve().parent
-        self.images_dir = self.nb_dir / (self.filename.stem.replace(' ', '_') + '_files')
+        self.nb_home = self.filename.parent
+        self.image_dir_name = self.title + '_files'
+        self.resources = self.get_resources()
+        self.nb = self.get_notebook()
         
         self.headers = self.get_headers()
         self.author_id = self.get_author_id()
         self.pub_id = self.get_pub_id()
-        if dataframe_image:
-            self.create_markdown()
-        self.md = self.get_markdown()
+        self.md, self.image_data_dict = self.create_markdown()
         self.load_images_to_medium()
         self.publish_to_medium()
 
+    def get_resources(self):
+        resources = {'metadata': {'path': str(self.nb_home), 
+                                  'name': self.title},
+                     'output_files_dir': self.image_dir_name}
+        return resources
+
+    def get_notebook(self):
+        with open(self.filename) as f:
+            nb = nbformat.read(f, as_version=4)
+        return nb
+
     def get_integration_token(self, it):
         if not it:
             it_path = Path.home() / '.jupyter_to_medium' / 'integration_token'
             it = open(it_path).read().splitlines()[0]
-        print(it)
         return it
 
     def get_headers(self):
@@ -70,41 +87,45 @@ class Publish:
                          f'Here is the data returned {data}')
 
     def create_markdown(self):
-        from dataframe_image import convert
-        convert(self.filename, to='md')
+        output_dir = self.nb_home / self.image_dir_name
+        if not output_dir.exists():
+            output_dir.mkdir()
+        print('output dir is', output_dir)
+        print('image_dir_name is', self.image_dir_name)
+        mp = MarkdownPreprocessor(output_dir=output_dir,
+                                  image_dir_name=Path(self.image_dir_name))
+        self.nb, self.resources = mp.preprocess(self.nb, self.resources)
 
-    def get_markdown(self):
-        p = self.nb_dir / f'{self.filename.stem}.md'
-        return open(p).read()
+        no_ex_pp = NoExecuteDataFramePreprocessor()
+        self.nb, self.resources = no_ex_pp.preprocess(self.nb, self.resources)
+
+        me = MarkdownExporter()
+        md, self.resources = me.from_notebook_node(self.nb, self.resources)
+        image_data_dict = self.resources['outputs']
+        image_data_dict = self.get_image_data(image_data_dict)
+        return md, image_data_dict
+
+    def get_image_data(self, image_data_dict):
+        image_dir = self.nb_home / self.image_dir_name
+        for file in image_dir.iterdir():
+            data = open(file, 'rb').read()
+            rel_fn = Path(self.image_dir_name) / file.name
+            image_data_dict[str(rel_fn)] = data
+        return image_data_dict
 
-    def get_image_files(self):
-        pat_inline = r'\!\[.*?\]\((.*?\.(?:gif|png|jpg|jpeg|tiff))'
-        pat_ref = r'\[.*?\]:\s*(.*?\.(?:gif|png|jpg|jpeg|tiff))'
-        inline_files = re.findall(pat_inline, self.md)
-        ref_files = re.findall(pat_ref, self.md)
-        print(inline_files)
-        image_files = []
-        for file in inline_files + ref_files:
-            p = Path(file.strip())
-            if (self.nb_dir / p).exists() and p not in image_files:
-                image_files.append(p)
-        return image_files
-        
     def load_images_to_medium(self):
         all_json = []
-        image_files = self.get_image_files()
-        print(image_files)
-        for file in image_files:
-            extension = file.suffix[1:].lower()
+        for file, data in self.image_data_dict.items():
+            fp = Path(file)
+            extension = fp.suffix[1:].lower()
             if extension in self.IMAGE_TYPES:
                 print('loading image to medium')
-                name = file.stem
-                fileobj = (self.nb_dir / file).open(mode='rb')
-                file_payload = {'image': (name, fileobj, f'image/{extension}')}
+                name = fp.stem
+                file_payload = {'image': (name, data, f'image/{extension}')}
                 r = requests.post(self.IMAGE_URL, headers=self.headers, files=file_payload)
                 req_json = r.json()
                 new_url = req_json['data']['url']
-                self.md = self.md.replace(str(file), new_url)
+                self.md = self.md.replace(urllib.parse.quote(file), new_url)
                 all_json.append(req_json)
         
         with open('medium_images_data_report.json', 'w') as f:
@@ -112,7 +133,7 @@ class Publish:
 
         # updated markdown
         new_md_filename = self.filename.stem + '_medium.md'
-        with open(self.nb_dir / new_md_filename, 'w') as f:
+        with open(self.nb_home / new_md_filename, 'w') as f:
             f.write(self.md)
 
     def publish_to_medium(self):
@@ -136,9 +157,10 @@ class Publish:
         self.result = requests.post(post_url, headers=self.headers, json=json_data)
         
 
-def publish(filename, integration_token=None, pub_name=None, dataframe_image=True, 
-            title=None, tags=None, publish_status='draft', notify_followers=False, 
-            license='all-rights-reserved', canonical_url=None):
+def publish(filename, integration_token=None, pub_name=None, title=None, 
+            tags=None, publish_status='draft', notify_followers=False, 
+            license='all-rights-reserved', canonical_url=None, chrome_path=None,
+            download_markdown=False):
     '''
     Publish a Jupyter Notebook directly to Medium as a blog post.
 
@@ -153,22 +175,13 @@ def publish(filename, integration_token=None, pub_name=None, dataframe_image=Tru
         '.jupyter_to_medium/integration_token'. You'll need to create 
         this directory and file first and paste your token there.
 
-        Otherwise, you can pass the integration token directly as a 
-        string to this parameter.
+        Otherwise, pass the integration token directly as a string.
+
+        Learn how to get an integration token from Medium.
+        https://github.com/Medium/medium-api-docs
     
     pub_name : str, default `None`
         Name of Medium publication. Not necessary if publishing as a user.
-    
-    dataframe_image : bool, default `True`
-        Use package dataframe_image to convert the notebook to markdown.
-        If you set this as `False`, then you must have your notebook 
-        already converted to markdown.
-        
-        Using dataframe_image converts pandas DataFrames to images, which
-        is necessary when publishing to Medium. A new markdown file will 
-        be created in the same directory as your notebook along with 
-        another direcory named {notebook_name}_files containing all
-        the DataFrame images (if you have any).
         
     title : str, default `None`
         Title of the Medium post. Leave as `None` to use the name 
@@ -181,7 +194,7 @@ def publish(filename, integration_token=None, pub_name=None, dataframe_image=Tru
     publish_status : str, default 'draft'
         The status of the post. Valid values are 'public', 'draft', or 'unlisted'.
     
-    notify_followers : bool, default False
+    notify_followers : bool, default `False`
         Whether to notify followers that the user has published.
     
     license : str, default 'all-rights-reserved'
@@ -192,7 +205,17 @@ def publish(filename, integration_token=None, pub_name=None, dataframe_image=Tru
     canonical_url : str, default `None`
         A URL of the original home of this content, if it was originally 
         published elsewhere.
+
+    chrome_path : str, default `None`
+        Path to your machine's chrome executable. By default, it is 
+        automatically found. Use this when chrome is not automatically found.
+
+    download_markdown : bool, default `False`
+        Whether or not to download the markdown and corresponding image files. They 
+        will be placed in the same folder containing the notebook. The images will be
+        in a folder with _files appended to it.
     '''
-    p = Publish(filename, integration_token, pub_name, dataframe_image, title, 
-                tags, publish_status, notify_followers, license, canonical_url)
+    p = Publish(filename, integration_token, pub_name, title, tags, 
+                publish_status, notify_followers, license, canonical_url,
+                chrome_path, download_markdown)
     return p.result.json()
\ No newline at end of file
